
#pragma once

#include <tee_session/client.h>
#include <base/connection.h>
#include <base/allocator.h>

namespace Tee {
	enum class Child_connections { NONE, ONE };

	struct Connection;
}

namespace detail {

	using Genode::size_t;

	// unnamed namespace to avoid linking issues
	namespace
	{

		// we will provide a memory for a connection to the child and for every child
		// connection so, if there is child will have no connections, we only need a RAM
		// for one connectionm if child will have one connection, we will need a RAM for
		// two connections
		size_t _mult(Tee::Child_connections child_connections) {
			return child_connections == Tee::Child_connections::ONE ? 2u : 1u;
		}

		size_t _ram_quota(size_t tx_buf_size, size_t rx_buf_size, Tee::Child_connections child_connections)
		{
			return _mult(child_connections) * (32 * 1024 * sizeof(long) + tx_buf_size + rx_buf_size);
		}
	}

} // namespace detail


struct Tee::Connection : Genode::Connection<Session>, Session_client
{
	/**
	 * Constructor
	 *
	 * \param tx_buffer_alloc  allocator used for managing the
	 *                         transmission buffer
	 * \param tx_buf_size      size of transmission buffer in bytes
	 * \param rx_buf_size      size of reception buffer in bytes
	 */
	Connection(Genode::Env             &env,
	           Genode::Range_allocator *tx_block_alloc,
	           Genode::size_t           tx_buf_size,
	           Genode::size_t           rx_buf_size,
	           char const              *label             = "",
	           Tee::Child_connections   child_connections = Tee::Child_connections::NONE)
	:
		Genode::Connection<Session> { env,
		                              label,
		                              Ram_quota { detail::_ram_quota(tx_buf_size, rx_buf_size, child_connections) },
		                              Args { "tx_buf_size=", tx_buf_size, ", "
		                                     "rx_buf_size=", rx_buf_size} },
		Session_client(cap(), *tx_block_alloc, env.rm())
	{ }

	void sigh_tx_channel(Genode::Signal_context_capability sigh)
	{
		Session_client::tx_channel()->sigh_ack_avail(sigh);
	}

	void sigh_rx_channel(Genode::Signal_context_capability sigh)
	{
		Session_client::rx_channel()->sigh_packet_avail(sigh);
	}

};

